/******************************************************************************
** Az albbi program s informcik a Nemzeti Kutatsi Fejlesztsi Programok
** 2002. vi plyzatn a 2. sz. programban elfogadott NKFP-2/0017/2002 szm
** ,,Adatrosta - webtrhzak nagytmeg adatbzisainak elemzse adatbnyszati 
** s statisztikai eszkzk segtsgvel'' elnevezs projekt termke.
**
** A program a projekt folyamn feldolgozsra kerl naplllomnyok
** (web, access s mail) binris kdolst s tmrtst vgzi.
**
** A program s informcik az Adatrosta projekt konzorciumnak tulajdona.
** Engedly nlkli felhasznlsa tilos.
**
** Az Adatrosta projekt konzorcium tagjai:
** ELTE TTK Szmtgptudomnyi Tsz.
** BME Matematika Intzet
** MTA SZTAKI Informatikai Kutatlaboratrium
** Axelero Rt.
** econet.hu Rt.
**
** http://adatrosta.hu/
** email: info@adatrosta.hu
**
** (C) 2003. Minden jog fenntartva.
**
**
** This data and information are property of the "Data Riddle" consortium.
** Modification or use without permission is strictly prohibited.
**
** (C) 2003 All rights reserved.
**
** Author: RACZ Balazs, 2003.
**
******************************************************************************/

/*!
  \file   log.h
  \brief  logging of status, warnings and error messages

  Provides routines for displaying and/or logging status, giving warnings.

  Set DEBUG_LEVEL to your needs with compiler option or rewrite debug
  information processing. There is a slot to provide debug level
  selection based on the message source (domain), which is currently
  __FILE__. This is yet unused (i.e. constant), you may set the debug
  level based on the source file.
  
  \author RACZ Balazs
  \date 2003-01-19 */

#ifndef __LOG_H__
#define __LOG_H__

//common interface

//! append to log, level ERR+\p level, \p format and \p args... like in printf()
#define err( level, format, arg... )  LOG( "31", __FILE__ , LEVEL_ERR+level, "E", format, ##arg )
//! append to log, level WARN+\p level, \p format and \p args... like in printf()
#define warn( level, format, arg... )  LOG( "33", __FILE__ , LEVEL_WARN+level, "W", format, ##arg )
//! append to log, level PERF+\p level, \p format and \p args... like in printf()
#define perf( level, format, arg... )  LOG( "0", __FILE__ , LEVEL_PERF+level, "p", format, ##arg )
//! append to log, level STATUS+\p level, \p format and \p args... like in printf()
#define status( level, format, arg... )  LOG( "0", __FILE__ , LEVEL_STATUS+level, "s",  format, ##arg )
//! append to log, level INFO+\p level, \p format and \p args... like in printf()
#define info( level, format, arg... )  LOG( "0", __FILE__ , LEVEL_INFO+level, "i",  format, ##arg )
//! append to log, level DEBUG+\p level, \p format and \p args... like in printf()
#define dbg( level, format, arg... )  LOG( "0", __FILE__ , LEVEL_DBG+level, "d", format, ##arg )


//! Base debug level of errors
#define LEVEL_ERR	0
//! Base debug level of warnings
#define LEVEL_WARN	10
//! Base level of performance (statistics) messages
#define LEVEL_PERF	20
//! Base debug level of status messages
#define LEVEL_STATUS	30
//! Base debug level of informative messages
#define LEVEL_INFO	40
//! Base debug level of debugging messages
#define LEVEL_DBG	50


//implementation

#include <stdio.h>

//! \brief append to log (generic)
/** 
    \param domain is a string literal, specifies the source of the message

    \param level is a number, the greater, the less important the log
    entry. loglevel specification will filter levels to less than the
    specified number.
    
    \param levelchr is a string literal denoting the level.
    \param format is a printf() like format string
    \param arg are the arguments to the format string.
*/
#define LOG( color, domain, level, levelchr, format, arg... ) {if (level<=debug_level(domain)) { fprintf(stderr, "\033[" color "m" domain "(" levelchr "): " format "\033[0m\n", ## arg); fflush(stderr); }} 

//! \brief defines the debug level as a function of the domain.
/** currently constant
 */
#define debug_level( x )  DEBUG_LEVEL

#ifndef DEBUG_LEVEL
#warning You should define DEBUG_LEVEL or expect full debugging output everywhere
//! You should define DEBUG_LEVEL externally (makefile, perhaps).
#define DEBUG_LEVEL 255
#endif



#endif //__LOG_H__



